import numpy as np


def if_valid_partition(row_boundaries, col_boundaries, params):
    H = params['H']
    W = params['W']
    k = params['num_cars']

    flight_time_factor = params['flight_time_factor']
    comp_time_factor = params['comp_time_factor']
    trans_time_factor = params['trans_time_factor']
    car_time_factor = params['car_time_factor']
    bs_time_factor = params['bs_time_factor']

    flight_energy_factor = params['flight_energy_factor']
    comp_energy_factor = params['comp_energy_factor']
    trans_energy_factor = params['trans_energy_factor']
    battery_energy_capacity = params['battery_energy_capacity']

    # 根据分割边界生成所有矩形任务
    rectangles = []
    for i in range(len(row_boundaries) - 1):
        for j in range(len(col_boundaries) - 1):
            r1 = row_boundaries[i]
            r2 = row_boundaries[i + 1]
            c1 = col_boundaries[j]
            c2 = col_boundaries[j + 1]
            d = (r2 - r1) * H * (c2 - c1) * W  # 任务的照片数量（矩形面积）

            # 求解rho
            rho_time_limit = (flight_time_factor - trans_time_factor) / \
                (comp_time_factor - trans_time_factor)
            rho_energy_limit = (battery_energy_capacity - flight_energy_factor * d - trans_energy_factor * d) / \
                (comp_energy_factor * d - trans_energy_factor * d)
            if rho_energy_limit < 0:
                return []

            rho = min(rho_time_limit, rho_energy_limit)
            flight_time = flight_time_factor * d
            comp_time = comp_time_factor * rho * d
            trans_time = trans_time_factor * (1 - rho) * d
            bs_time = bs_time_factor * (1 - rho) * d

            # 计算任务矩形中心，用于后续车辆移动时间计算
            center_r = (r1 + r2) / 2.0 * H
            center_c = (c1 + c2) / 2.0 * W

            rectangles.append({
                # 'r1': r1, 'r2': r2, 'c1': c1, 'c2': c2,
                'd': d,
                'rho': rho,
                'flight_time': flight_time,
                'comp_time': comp_time,
                'trans_time': trans_time,
                'bs_time': bs_time,
                'center': (center_r, center_c)
            })
    return rectangles
